/*
* Copyright (c) 2008-2009 Christian Buchner <Christian.Buchner@gmail.com>
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*     * Redistributions of source code must retain the above copyright
*       notice, this list of conditions and the following disclaimer.
*     * Redistributions in binary form must reproduce the above copyright
*       notice, this list of conditions and the following disclaimer in the
*       documentation and/or other materials provided with the distribution.
*
* THIS SOFTWARE IS PROVIDED BY Christian Buchner ''AS IS'' AND ANY 
* EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
* DISCLAIMED. IN NO EVENT SHALL Christian Buchner BE LIABLE FOR ANY
* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef CUDATYPES_H
#define CUDATYPES_H

#include "cudaconventions.h"  // define my own CUDA coding conventions

/**
 * Unsigned 24 bit integer type for use in array index calculations.
 * No clamping to 24 bit results occurs except for operands of a multiplication.
 * It's the responsibility of the user of the class to verify that operands
 * for multiplications are always within the 24 bit range.
 */
typedef struct _uint24
{
    int value;

    _uint24(unsigned int a) {
        value = a;
    }

    operator unsigned int () const {
        return value;
    }

    M_DEVICE _uint24 operator*(const _uint24 REF(b)) const {
        return (_uint24)__umul24(value, b.value);
    };

    M_DEVICE _uint24 operator*=(const _uint24 REF(b)) {
        value = __umul24(value, b.value);
        return *this;
    };
} uint24;


/**
 * Signed 24 bit integer type for use in array index calculations.
 * No clamping to 24 bit results occurs except for operands of a multiplication.
 * It's the responsibility of the user of the class to verify that operands
 * for multiplications are always within the 24 bit range.
 */
typedef struct _int24
{
    int value;

    _int24(int a) {
        value = a;
    }

    operator int () const {
        return value;
    }

    M_DEVICE _int24 operator*(const _int24 REF(b)) const {
        return (_int24)__mul24(value, b.value);
    };

    M_DEVICE _int24 operator*=(const _int24 REF(b)) {
        value = __mul24(value, b.value);
        return *this;
    };

} int24;

#endif // #ifndef CUDATYPES_H
