/*
* Copyright (c) 2008-2009 Christian Buchner <Christian.Buchner@gmail.com>
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*     * Redistributions of source code must retain the above copyright
*       notice, this list of conditions and the following disclaimer.
*     * Redistributions in binary form must reproduce the above copyright
*       notice, this list of conditions and the following disclaimer in the
*       documentation and/or other materials provided with the distribution.
*
* THIS SOFTWARE IS PROVIDED BY Christian Buchner ''AS IS'' AND ANY 
* EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
* DISCLAIMED. IN NO EVENT SHALL Christian Buchner BE LIABLE FOR ANY
* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef DOUBLESINGLE_H
#define DOUBLESINGLE_H

#include <stdint.h>

#ifdef __CUDACC__
#include <device_functions.h>
#include <math_functions.h>

#define __fmul_rn(x,y) ((x)*(y))  // dirty hack to allow __host__ __device__ * and / operators to work

//#define __fmul_rn(x,y) __myfmul_rn(x,y)
extern __host__ __device__ float                  __myfmul_rn(float a, float b);
#if defined(__CUDABE__)
__device_func__(float __myfmul_rn(const float &a, const float &b))
{
  __internal_fmul_kernel(a, b, cudaRoundNearest);
}
#endif

#else
#define __fmul_rn(x,y) ((x)*(y))
#endif

#include "dsmath.h"

/**
 * An object oriented wrapper around functions provided by "dsmath.h" header file
 * http://forums.nvidia.com/index.php?showtopic=73067&pid=414254&mode=threaded&show=&st=0
 */
typedef struct _doublesingle
{
    float2 value;
    
    _doublesingle() {
        // makes go away some compiler warnings due to unfinished implementations below
//        value.x = value.y = 0.0;
    }

    M_HOST _doublesingle(const double REF(a)) {
        set(value, a);
    }

    M_HOST _doublesingle(const float2 REF(a)) {
        set(value, a);
    }

    M_HOSTDEVICE _doublesingle operator+(const _doublesingle REF(b)) const {
        _doublesingle result = (value + b.value);
        return result;
    }

    M_HOSTDEVICE _doublesingle operator-(const _doublesingle REF(b)) const {
        _doublesingle result = (value - b.value);
        return result;
    }

    M_HOSTDEVICE _doublesingle operator*(const _doublesingle REF(b)) const {
        _doublesingle result = (value * b.value);
        return result;
    }

    M_HOSTDEVICE _doublesingle operator/(const _doublesingle REF(b)) const {
        _doublesingle result = (value / b.value);
        return result;
    }

    M_HOSTDEVICE _doublesingle operator-() const {
        _doublesingle result = (-value);
        return result;
    }

} doublesingle;


/**
 * A float2/double2 compatible vector storage object to enable
 * doublesingle objects being used with my cudacomplex type.
 */
typedef struct _doublesingle2
{
    doublesingle x;
    doublesingle y;
    
} doublesingle2;

#endif // #ifndef DOUBLESINGLE_H
